@@ -1,11 +1,13 @@
 #include <glib/gtypes.h>
 
 double
-rsvg_css_parse_length (const char *str, gdouble pixels_per_inch, int *fixed);
+rsvg_css_parse_length (const char *str, gdouble pixels_per_inch, 
+		       gint *percent, gint *em, gint *ex);
 
 double
 rsvg_css_parse_normalized_length(const char *str, gdouble pixels_per_inch,
-				 gdouble normalize_to);
+				 gdouble width_or_height, gdouble font_size,
+				 gdouble x_height);
 
 gboolean
 rsvg_css_param_match (const char *str, const char *param_name);
@@ -18,6 +20,3 @@ rsvg_css_parse_color (const char *str);
 
 guint
 rsvg_css_parse_opacity (const char *str);
-
-double
-rsvg_css_parse_fontsize (const char *str);
