@@ -9,7 +9,8 @@ typedef enum {
      ie specific definable things? For now, we're going low level,
      but it's not clear that's the best way to go. */
   RSVG_DEF_LINGRAD,
-  RSVG_DEF_RADGRAD
+  RSVG_DEF_RADGRAD,
+  RSVG_DEF_PATTERN
 } RsvgDefType;
 
 struct _RsvgDefVal {
