@@ -4,16 +4,16 @@
    Copyright (C) 2000 Eazel, Inc.
   
    This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License as
+   modify it under the terms of the GNU Library General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
+   Library General Public License for more details.
   
-   You should have received a copy of the GNU General Public
+   You should have received a copy of the GNU Library General Public
    License along with this program; if not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
@@ -21,22 +21,16 @@
    Author: Raph Levien <raph@artofcode.com>
 */
 
-#include <string.h>
-#include <ctype.h>
-
-#include <glib.h>
-
-#include <libart_lgpl/art_misc.h>
-#include <libart_lgpl/art_alphagamma.h>
-#include <libart_lgpl/art_filterlevel.h>
-#include <libart_lgpl/art_affine.h>
-#include "art_render.h"
-#include "art_render_gradient.h"
-
-#include "rsvg-css.h"
-#include "rsvg-defs.h"
+#include "config.h"
 #include "rsvg-paint-server.h"
 
+#include <glib/gmem.h>
+#include <glib/gmessages.h>
+#include <glib/gstrfuncs.h>
+#include <libart_lgpl/art_affine.h>
+#include <string.h>
+#include "rsvg-css.h"
+
 typedef struct _RsvgPaintServerSolid RsvgPaintServerSolid;
 typedef struct _RsvgPaintServerLinGrad RsvgPaintServerLinGrad;
 typedef struct _RsvgPaintServerRadGrad RsvgPaintServerRadGrad;
@@ -286,7 +280,7 @@ rsvg_paint_server_parse (const RsvgDefs *defs, const char *str)
       char *name;
       RsvgDefVal *val;
 
-      while (isspace (*p)) p++;
+      while (g_ascii_isspace (*p)) p++;
       if (*p != '#')
 	return NULL;
       p++;
@@ -327,6 +321,7 @@ void
 rsvg_render_paint_server (ArtRender *ar, RsvgPaintServer *ps,
 			  const RsvgPSCtx *ctx)
 {
+  g_return_if_fail (ar != NULL);
   if (ps != NULL)
     ps->render (ps, ar, ctx);
 }
@@ -355,3 +350,59 @@ rsvg_paint_server_unref (RsvgPaintServer *ps)
   if (--ps->refcnt == 0)
     ps->free (ps);
 }
+
+RsvgRadialGradient *
+rsvg_clone_radial_gradient (const RsvgRadialGradient *grad)
+{
+  RsvgRadialGradient * clone = NULL;
+  int i;
+
+  clone = g_new (RsvgRadialGradient, 1);
+  clone->super.type = grad->super.type;
+  clone->super.free = grad->super.free;
+
+  for (i = 0; i < 6; i++)
+    clone->affine[i] = grad->affine[i];
+  clone->cx = grad->cx;
+  clone->cy = grad->cy;
+  clone->r  = grad->r;
+  clone->fx = grad->fx;
+  clone->fy = grad->fy;
+
+  clone->stops = g_new (RsvgGradientStops, 1);
+  clone->stops->n_stop = grad->stops->n_stop;
+  clone->stops->stop = g_new (RsvgGradientStop, clone->stops->n_stop);
+
+  for (i = 0; i < grad->stops->n_stop; i++)
+    clone->stops->stop[i] = grad->stops->stop[i];
+
+  return clone;
+}
+
+RsvgLinearGradient *
+rsvg_clone_linear_gradient (const RsvgLinearGradient *grad)
+{
+  RsvgLinearGradient * clone = NULL;
+  int i;
+
+  clone = g_new (RsvgLinearGradient, 1);
+  clone->super.type = grad->super.type;
+  clone->super.free = grad->super.free;
+
+  for (i = 0; i < 6; i++)
+    clone->affine[i] = grad->affine[i];
+  clone->x1 = grad->x1;
+  clone->y1 = grad->y1;
+  clone->x2 = grad->x2;
+  clone->y2 = grad->y2;
+  clone->spread = grad->spread;
+
+  clone->stops = g_new (RsvgGradientStops, 1);
+  clone->stops->n_stop = grad->stops->n_stop;
+  clone->stops->stop = g_new (RsvgGradientStop, clone->stops->n_stop);
+
+  for (i = 0; i < grad->stops->n_stop; i++)
+    clone->stops->stop[i] = grad->stops->stop[i];
+
+  return clone;
+}
